﻿using System;
using System.Configuration;
using System.Linq;
using MongoDB.Driver;
using MongoDB.Driver.Builders;
using MongoDB.Driver.Linq;
using MongoSamples.Model;
using NUnit.Framework;

namespace MongoSamples
{
    /// <summary>
    /// Tests to demonstrate LINQ support of the MongoDB C# driver.
    /// </summary>
    /// <remarks>See http://docs.mongodb.org/ecosystem/tutorial/use-linq-queries-with-csharp-driver/ for details on
    /// supported query operators.</remarks>
    [TestFixture]
    class LinqQueryTests
    {
        private readonly MongoClient _client;

        public LinqQueryTests()
        {
            _client = GetMongoClient();
        }

        [Test]
        public void Test_GetDogImages()
        {
            MongoDatabase db = GetDatabase("gallery");
            MongoCollection<Image> collection = db.GetCollection<Image>("images");

            // Shell: db.images.find({tags:"dogs"})
            var query = (from img in collection.AsQueryable()
                         where img.Tags.Contains("dogs")
                         select img).Take(10);

            foreach (Image image in query)
            {
                Assert.IsNotNull(image);
                Assert.IsTrue(image.Tags.Contains("dogs"));
            }
        }

        [Test]
        public void Test_CountKittenImages()
        {
            MongoDatabase db = GetDatabase("gallery");
            MongoCollection<Image> collection = db.GetCollection<Image>("images");

            // Shell: db.images.find({tags:"kittens"}).count()
            int result = collection.AsQueryable().Count(img => img.Tags.Contains("kittens"));

            Assert.GreaterOrEqual(result, 0);
        }

        private MongoDatabase GetDatabase(string dbName)
        {
            return _client.GetServer().GetDatabase(dbName);
        }

        private static MongoClient GetMongoClient()
        {
            // See http://docs.mongodb.org/manual/reference/connection-string/ for full details
            // on MongoDB connection string format and options.
            const string connectionStringTemplate = "mongodb://{0}:27017";

            string host = ConfigurationManager.AppSettings["Mongo.Host"];
            string connString = String.Format(connectionStringTemplate, host);

            MongoClient client = new MongoClient(connString);

            return client;
        }
    }
}
